@@ -35,6 +35,7 @@
 #ifdef STDC_HEADERS
 #include <stdlib.h>
 #endif /* STDC_HEADERS */
+#include <errno.h>
 #include <fcntl.h>
 #ifdef HAVE_UNISTD_H
 #include <unistd.h>
@@ -60,7 +61,7 @@
 #endif
 
 #ifndef lint
-static const char rcsid[] = "$Id: random.c,v 1.2 2001/05/22 23:14:54 millert Exp $";
+static const char rcsid[] = "$Id: random.c,v 1.3 2001/06/27 03:39:53 millert Exp $";
 #endif /* not lint */
 
 #ifdef HAVE_SRANDOMDEV
@@ -70,6 +71,34 @@ seed_random()
 	srandomdev();
 }
 #else !HAVE_SRANDOMDEV
+#ifdef _PATH_RANDOM
+static ssize_t
+read_loop(fd, buf, count)
+	int fd;
+	void *buf;
+	size_t count;
+{
+	char *buffer = (char *) buf;
+	ssize_t offset, block;
+
+	offset = 0;
+	while (count > 0) {
+		block = read(fd, &buffer[offset], count);
+
+		if (block < 0) {
+			if (errno == EINTR || errno == EAGAIN)
+				continue;
+			return(block);
+		} else if (block == 0)
+			return(offset);
+
+		offset += block;
+		count -= block;
+	}
+
+	return(offset);
+}
+#endif /* _PATH_RANDOM */
 void
 seed_random()
 {
@@ -80,7 +109,7 @@ seed_random()
 	int fd;
 
 	if ((fd = open(_PATH_RANDOM, O_RDONLY)) != -1 &&
-	    read(fd, &seed, sizeof(seed)) == sizeof(seed)) {
+	    read_loop(fd, &seed, sizeof(seed)) == sizeof(seed)) {
 		close(fd);
 		SRAND(seed);
 		return;
@@ -101,8 +130,10 @@ get_random()
 {
 	static int initialized;
 
-	if (!initialized)
+	if (!initialized) {
 		seed_random();
+		initialized = 1;
+	}
 
 	return(RAND());
 }
